@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
+ * Joe <cuchac@email.cz>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +24,7 @@
 
 #include <string>
 #include <sstream>
+#include <stdint.h>
 
 #include "s11n.h"
 
@@ -53,6 +55,8 @@ public:
         set_video_stream,               // int
         cycle_audio_stream,             // no parameters
         set_audio_stream,               // int
+        cycle_subtitle_stream,          // no parameters
+        set_subtitle_stream,            // int
         set_stereo_layout,              // video_frame::stereo_layout, bool
         set_stereo_mode,                // parameters::stereo_mode, bool
         toggle_stereo_mode_swap,        // no parameters
@@ -66,13 +70,20 @@ public:
         set_hue,                        // float (absolute value)
         adjust_saturation,              // float (relative adjustment)
         set_saturation,                 // float (absolute value)
-        seek,                           // float (relative adjustment)
-        set_pos,                        // float (absolute position)
         adjust_parallax,                // float (relative adjustment)
         set_parallax,                   // float (absolute value)
         set_crosstalk,                  // 3 floats (absolute values)
         adjust_ghostbust,               // float (relative adjustment)
-        set_ghostbust                   // float (absolute value)
+        set_ghostbust,                  // float (absolute value)
+        set_subtitle_encoding,          // string (encoding name)
+        set_subtitle_font,              // string (font name)
+        set_subtitle_size,              // int
+        set_subtitle_scale,             // float
+        set_subtitle_color,             // uint64_t
+        adjust_subtitle_parallax,       // float (relative adjustment)
+        set_subtitle_parallax,          // float (absolute value)
+        seek,                           // float (relative adjustment)
+        set_pos                         // float (absolute position)
     };
     
     type type;
@@ -116,6 +127,7 @@ public:
         pause,                  // bool
         video_stream,           // int
         audio_stream,           // int
+        subtitle_stream,        // int
         stereo_layout,          // video_frame::stereo_layout, bool
         stereo_mode,            // parameters::stereo_mode, bool
         stereo_mode_swap,       // bool
@@ -125,10 +137,16 @@ public:
         brightness,             // float
         hue,                    // float
         saturation,             // float
-        pos,                    // float
         parallax,               // float
         crosstalk,              // 3 floats
-        ghostbust               // float
+        ghostbust,              // float
+        subtitle_encoding,      // string
+        subtitle_font,          // string
+        subtitle_size,          // int
+        subtitle_scale,         // float
+        subtitle_color,         // uint64_t
+        subtitle_parallax,      // float
+        pos                     // float
     };
     
     type type;
@@ -173,9 +191,26 @@ public:
         current = ossc.str();
     }
 
-    notification(enum type t, const std::string &p, const std::string &c) :
-        type(t), previous(p), current(c)
+    notification(enum type t, uint64_t p, uint64_t c) :
+        type(t)
     {
+        std::ostringstream ossp;
+        s11n::save(ossp, p);
+        previous = ossp.str();
+        std::ostringstream ossc;
+        s11n::save(ossc, c);
+        current = ossc.str();
+    }
+
+    notification(enum type t, const std::string &p, const std::string &c) :
+        type(t)
+    {
+        std::ostringstream ossp;
+        s11n::save(ossp, p);
+        previous = ossp.str();
+        std::ostringstream ossc;
+        s11n::save(ossc, c);
+        current = ossc.str();
     }
 };
 
