@@ -344,7 +344,8 @@ int main(int argc, char *argv[])
                     "  -b|--benchmark           Benchmark mode (no audio, show fps).\n"
                     "\n"
                     "Interactive control:\n"
-                    "  q or ESC                 Quit.\n"
+                    "  ESC                      Leave fullscreen mode, or quit.\n"
+                    "  q                        Quit.\n"
                     "  p or SPACE               Pause / unpause.\n"
                     "  f                        Toggle fullscreen.\n"
                     "  c                        Center window.\n"
