@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
+ * Joe <joe@wpj.cz>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -130,9 +131,12 @@ public:
         d64             // double
     } sample_format_t;
 
+    // Description of the content
+    std::string language;               // Language information (empty if unknown)
     int channels;                       // 1 (mono), 2 (stereo), 4 (quad), 6 (5:1), 7 (6:1), or 8 (7:1)
     int rate;                           // Samples per second
     sample_format_t sample_format;      // Sample format
+
     // The data. Note that an audio blob does not own the data stored in these pointers,
     // so it does not free them on destruction.
     void *data;                         // Pointer to the data
@@ -157,6 +161,82 @@ public:
     int sample_bits() const;
 };
 
+class subtitle_box : public s11n
+{
+public:
+    // Format
+    typedef enum
+    {
+        ass,            // Advanced SubStation Alpha (ASS) format
+        text,           // UTF-8 text
+        image           // Image in BGRA32 format, with box coordinates
+    } format_t;
+
+    // Image data
+    class image_t : public s11n
+    {
+    public:
+        int w, h;                       // Dimensions
+        int x, y;                       // Position w.r.t. the video frame
+        std::vector<uint8_t> palette;   // Palette, with R,G,B,A components for each palette entry.
+        std::vector<uint8_t> data;      // Bitmap using the palette
+        size_t linesize;                // Size of one bitmap line (may differ from width)
+
+        void save(std::ostream &os) const;
+        void load(std::istream &is);
+    };
+
+    // Description of the content
+    std::string language;               // Language information (empty if unknown)
+
+    // Data
+    format_t format;                    // Subtitle data format
+    std::string style;                  // Style info (only if format is ass)
+    std::string str;                    // Event text (only if format ass or text)
+    std::vector<image_t> images;        // Images. These need to be alpha-blended.
+
+    // Presentation time information
+    int64_t presentation_start_time;    // Presentation timestamp
+    int64_t presentation_stop_time;     // End of presentation timestamp
+
+    // Constructor, Destructor
+    subtitle_box();
+
+    // Comparison. This assumes that two subtitle boxes are identical when they
+    // have the same presentation times.
+    bool operator==(const subtitle_box &box) const
+    {
+        return (presentation_start_time == box.presentation_start_time
+                && presentation_stop_time == box.presentation_stop_time);
+    }
+    bool operator!=(const subtitle_box &box) const
+    {
+        return !operator==(box);
+    }
+
+    // Does this box contain valid data?
+    bool is_valid() const
+    {
+        return (((format == ass || format == text) && !str.empty())
+                || (format == image && !images.empty()));
+    }
+
+    // Does this box stay constant during its complete presentation time?
+    // (ASS subtitles may be animated and thus need to be rerendered when the clock changes)
+    bool is_constant() const
+    {
+        return (format != ass);
+    }
+
+    // Return a string describing the format
+    std::string format_info() const;    // Human readable information
+    std::string format_name() const;    // Short code
+
+    // Serialization
+    void save(std::ostream &os) const;
+    void load(std::istream &is);
+};
+
 class parameters : public s11n
 {
 public:
@@ -173,6 +253,7 @@ public:
         even_odd_rows,                  // Left view even rows, right view odd rows
         even_odd_columns,               // Left view even columns, right view odd columns
         checkerboard,                   // Checkerboard pattern
+        hdmi_frame_pack,                // HDMI Frame packing (top-bottom separated by 1/49 height)
         red_cyan_monochrome,            // Red/cyan anaglyph, monochrome method
         red_cyan_half_color,            // Red/cyan anaglyph, half color method
         red_cyan_full_color,            // Red/cyan anaglyph, full color method
@@ -200,13 +281,19 @@ public:
     float brightness;                   // Brightness adjustment, -1 .. +1
     float hue;                          // Hue adjustment, -1 .. +1
     float saturation;                   // Saturation adjustment, -1 .. +1
+    std::string subtitle_encoding;      // Subtitle encoding, empty means keep default
+    std::string subtitle_font;          // Subtitle font name, empty means keep default
+    int subtitle_size;                  // Subtitle point size, -1 means keep default
+    float subtitle_scale;               // Scale factor
+    uint64_t subtitle_color;            // Subtitle color in uint32_t bgra32 format, > UINT32_MAX means keep default
+    float subtitle_parallax;            // Subtitle parallax adjustment, -1 .. +1
 
     // Constructor
     parameters();
 
     // Set all uninitialised values to their defaults
     void set_defaults();
-    
+
     // Convert the stereo mode to and from a string representation
     static std::string stereo_mode_to_string(stereo_mode_t stereo_mode, bool stereo_mode_swap);
     static void stereo_mode_from_string(const std::string &s, stereo_mode_t &stereo_mode, bool &stereo_mode_swap);
