@@ -586,22 +586,28 @@ void media_object::set_audio_blob_template(int index)
     }
     audio_blob_template.channels = audio_codec_ctx->channels;
     audio_blob_template.rate = audio_codec_ctx->sample_rate;
-    if (audio_codec_ctx->sample_fmt == SAMPLE_FMT_U8)
+    if (audio_codec_ctx->sample_fmt == AV_SAMPLE_FMT_U8)
     {
         audio_blob_template.sample_format = audio_blob::u8;
     }
-    else if (audio_codec_ctx->sample_fmt == SAMPLE_FMT_S16)
+    else if (audio_codec_ctx->sample_fmt == AV_SAMPLE_FMT_S16)
     {
         audio_blob_template.sample_format = audio_blob::s16;
     }
-    else if (audio_codec_ctx->sample_fmt == SAMPLE_FMT_FLT)
+    else if (audio_codec_ctx->sample_fmt == AV_SAMPLE_FMT_FLT)
     {
         audio_blob_template.sample_format = audio_blob::f32;
     }
-    else if (audio_codec_ctx->sample_fmt == SAMPLE_FMT_DBL)
+    else if (audio_codec_ctx->sample_fmt == AV_SAMPLE_FMT_DBL)
     {
         audio_blob_template.sample_format = audio_blob::d64;
     }
+    else if (audio_codec_ctx->sample_fmt == AV_SAMPLE_FMT_S32
+            && sizeof(int32_t) == sizeof(float))
+    {
+        // we need to convert this to AV_SAMPLE_FMT_FLT after decoding
+        audio_blob_template.sample_format = audio_blob::f32;
+    }
     else
     {
         throw exc(str::asprintf(_("%s audio stream %d: Cannot handle audio with sample format %s."),
@@ -658,7 +664,7 @@ void media_object::open(const std::string &url)
             && i < static_cast<unsigned int>(std::numeric_limits<int>::max()); i++)
     {
         _ffmpeg->format_ctx->streams[i]->discard = AVDISCARD_ALL;        // ignore by default; user must activate streams
-        if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == CODEC_TYPE_VIDEO)
+        if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO)
         {
             _ffmpeg->video_streams.push_back(i);
             int j = _ffmpeg->video_streams.size() - 1;
@@ -726,7 +732,7 @@ void media_object::open(const std::string &url)
             }
             _ffmpeg->video_last_timestamps.push_back(std::numeric_limits<int64_t>::min());
         }
-        else if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == CODEC_TYPE_AUDIO)
+        else if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO)
         {
             _ffmpeg->audio_streams.push_back(i);
             int j = _ffmpeg->audio_streams.size() - 1;
@@ -758,7 +764,7 @@ void media_object::open(const std::string &url)
             _ffmpeg->audio_buffers.push_back(std::vector<unsigned char>());
             _ffmpeg->audio_last_timestamps.push_back(std::numeric_limits<int64_t>::min());
         }
-        else if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == CODEC_TYPE_SUBTITLE)
+        else if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_SUBTITLE)
         {
             _ffmpeg->subtitle_streams.push_back(i);
             int j = _ffmpeg->subtitle_streams.size() - 1;
@@ -1385,6 +1391,23 @@ void audio_decode_thread::run()
                     continue;
                 }
                 // Put it in the decoded audio data buffer
+                if (_ffmpeg->audio_codec_ctxs[_audio_stream]->sample_fmt == AV_SAMPLE_FMT_S32)
+                {
+                    // we need to convert this to AV_SAMPLE_FMT_FLT
+                    assert(sizeof(int32_t) == sizeof(float));
+                    assert(tmpbuf_size % sizeof(int32_t) == 0);
+                    void *tmpbuf_v = static_cast<void *>(&(_ffmpeg->audio_tmpbufs[_audio_stream][0]));
+                    int32_t *tmpbuf_i32 = static_cast<int32_t *>(tmpbuf_v);
+                    float *tmpbuf_flt = static_cast<float *>(tmpbuf_v);
+                    const float posdiv = +static_cast<float>(std::numeric_limits<int32_t>::max());
+                    const float negdiv = -static_cast<float>(std::numeric_limits<int32_t>::min());
+                    for (size_t j = 0; j < tmpbuf_size / sizeof(int32_t); j++)
+                    {
+                        int32_t sample_i32 = tmpbuf_i32[j];
+                        float sample_flt = sample_i32 / (sample_i32 >= 0 ? posdiv : negdiv);
+                        tmpbuf_flt[j] = sample_flt;
+                    }
+                }
                 size_t old_size = _ffmpeg->audio_buffers[_audio_stream].size();
                 _ffmpeg->audio_buffers[_audio_stream].resize(old_size + tmpbuf_size);
                 memcpy(&(_ffmpeg->audio_buffers[_audio_stream][old_size]), _ffmpeg->audio_tmpbufs[_audio_stream], tmpbuf_size);
