@@ -4,6 +4,7 @@
  * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
  * Frédéric Devernay <frederic.devernay@inrialpes.fr>
+ * Joe <cuchac@email.cz>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +22,15 @@
 
 #include "config.h"
 
+#include <limits>
 #include <cstdlib>
 #include <cmath>
 
 #include <GL/glew.h>
 
+#include "gettext.h"
+#define _(string) gettext(string)
+
 #include "exc.h"
 #include "msg.h"
 #include "str.h"
@@ -88,6 +93,7 @@ video_output::video_output(bool receive_notifications) :
     _srgb_textures_are_broken = std::getenv("SRGB_TEXTURES_ARE_BROKEN");
 
     _input_pbo = 0;
+    _input_fbo = 0;
     _active_index = 1;
     for (int i = 0; i < 2; i++)
     {
@@ -99,10 +105,15 @@ video_output::video_output(bool receive_notifications) :
             _input_bgra32_tex[i][j] = 0;
         }
         _color_srgb_tex[i] = 0;
+        _input_subtitle_tex[i] = 0;
+        _input_subtitle_width[i] = -1;
+        _input_subtitle_height[i] = -1;
+        _input_subtitle_time[i] = std::numeric_limits<int64_t>::min();
     }
     _color_prg = 0;
     _color_fbo = 0;
     _render_prg = 0;
+    _render_dummy_tex = 0;
     _render_mask_tex = 0;
 }
 
@@ -134,20 +145,6 @@ void video_output::deinit()
     }
 }
 
-float video_output::screen_pixel_aspect_ratio()
-{
-    float screen_pixel_ar = screen_aspect_ratio()
-        * static_cast<float>(screen_height()) / static_cast<float>(screen_width());
-    if (std::fabs(screen_pixel_ar - 1.0f) < 0.03f)
-    {
-        // This screen most probably has square pixels, and the difference to 1.0
-        // is only due to slightly inaccurate measurements and rounding. Force
-        // 1.0 so that the user gets expected results.
-        screen_pixel_ar = 1.0f;
-    }
-    return screen_pixel_ar;
-}
-
 void video_output::set_suitable_size(int w, int h, float ar, parameters::stereo_mode_t stereo_mode)
 {
     int width = w;
@@ -156,7 +153,8 @@ void video_output::set_suitable_size(int w, int h, float ar, parameters::stereo_
     {
         w *= 2;
     }
-    else if (stereo_mode == parameters::top_bottom)
+    else if (stereo_mode == parameters::top_bottom
+            || stereo_mode == parameters::hdmi_frame_pack)
     {
         h *= 2;
     }
@@ -165,7 +163,8 @@ void video_output::set_suitable_size(int w, int h, float ar, parameters::stereo_
     {
         aspect_ratio /= 2.0f;
     }
-    else if (stereo_mode == parameters::top_bottom)
+    else if (stereo_mode == parameters::top_bottom
+            || stereo_mode == parameters::hdmi_frame_pack)
     {
         aspect_ratio *= 2.0f;
     }
@@ -194,6 +193,7 @@ void video_output::input_init(int index, const video_frame &frame)
 {
     assert(xgl::CheckError(HERE));
     glGenBuffers(1, &_input_pbo);
+    glGenFramebuffersEXT(1, &_input_fbo);
     if (frame.layout == video_frame::bgra32)
     {
         for (int i = 0; i < (frame.stereo_layout == video_frame::mono ? 1 : 2); i++)
@@ -277,6 +277,8 @@ void video_output::input_deinit(int index)
     assert(xgl::CheckError(HERE));
     glDeleteBuffers(1, &_input_pbo);
     _input_pbo = 0;
+    glDeleteFramebuffersEXT(1, &_input_fbo);
+    _input_fbo = 0;
     for (int i = 0; i < 2; i++)
     {
         if (_input_yuv_y_tex[index][i] != 0)
@@ -299,6 +301,15 @@ void video_output::input_deinit(int index)
             glDeleteTextures(1, &(_input_bgra32_tex[index][i]));
             _input_bgra32_tex[index][i] = 0;
         }
+        if (_input_subtitle_tex[i] != 0)
+        {
+            glDeleteTextures(1, &(_input_subtitle_tex[i]));
+            _input_subtitle_tex[i] = 0;
+        }
+        _input_subtitle_box[i] = subtitle_box();
+        _input_subtitle_width[i] = -1;
+        _input_subtitle_height[i] = -1;
+        _input_subtitle_time[i] = std::numeric_limits<int64_t>::min();
     }
     _input_yuv_chroma_width_divisor[index] = 0;
     _input_yuv_chroma_height_divisor[index] = 0;
@@ -311,7 +322,7 @@ static int next_multiple_of_4(int x)
     return (x / 4 + (x % 4 == 0 ? 0 : 1)) * 4;
 }
 
-void video_output::prepare_next_frame(const video_frame &frame)
+void video_output::prepare_next_frame(const video_frame &frame, const subtitle_box &subtitle)
 {
     assert(xgl::CheckError(HERE));
     int index = (_active_index == 0 ? 1 : 0);
@@ -325,8 +336,8 @@ void video_output::prepare_next_frame(const video_frame &frame)
     {
         input_deinit(index);
         input_init(index, frame);
-        _frame[index] = frame;
     }
+    _frame[index] = frame;
     int bytes_per_pixel = (frame.layout == video_frame::bgra32 ? 4 : 1);
     GLenum format = (frame.layout == video_frame::bgra32 ? GL_BGRA : GL_LUMINANCE);
     GLenum type = (frame.layout == video_frame::bgra32 ? GL_UNSIGNED_INT_8_8_8_8_REV : GL_UNSIGNED_BYTE);
@@ -361,7 +372,7 @@ void video_output::prepare_next_frame(const video_frame &frame)
             void *pboptr = glMapBuffer(GL_PIXEL_UNPACK_BUFFER, GL_WRITE_ONLY);
             if (!pboptr)
             {
-                msg::err("Cannot create a PBO buffer.");
+                msg::err(_("Cannot create a PBO buffer."));
                 abort();
             }
             assert(reinterpret_cast<uintptr_t>(pboptr) % 4 == 0);
@@ -380,6 +391,120 @@ void video_output::prepare_next_frame(const video_frame &frame)
         }
     }
     assert(xgl::CheckError(HERE));
+    // In the common case, the video display width and height do not change
+    // between preparing a frame and rendering it, so it is benefical to update
+    // to subtitle texture in this function (because other threads can do other
+    // work in parallel).
+    update_subtitle_tex(index, frame, subtitle, _params);
+}
+
+int video_output::video_display_width()
+{
+    assert(_viewport[0][2] > 0);
+    return _viewport[0][2];
+}
+
+int video_output::video_display_height()
+{
+    assert(_viewport[0][3] > 0);
+    return _viewport[0][3];
+}
+
+void video_output::update_subtitle_tex(int index, const video_frame &frame, const subtitle_box &subtitle, const parameters &params)
+{
+    assert(xgl::CheckError(HERE));
+    int width = 0;
+    int height = 0;
+    if (subtitle.is_valid())
+    {
+        if (_subtitle_renderer.render_to_display_size(subtitle))
+        {
+            width = video_display_width();
+            height = video_display_height();
+        }
+        else
+        {
+            width = frame.width;
+            height = frame.height;
+        }
+    }
+    if (subtitle.is_valid()
+            && (subtitle != _input_subtitle_box[index]
+                || (!subtitle.is_constant() && frame.presentation_time != _input_subtitle_time[index])
+                || width != _input_subtitle_width[index]
+                || height != _input_subtitle_height[index]
+                || params.subtitle_encoding != _input_subtitle_params.subtitle_encoding
+                || params.subtitle_font != _input_subtitle_params.subtitle_font
+                || params.subtitle_size != _input_subtitle_params.subtitle_size
+                || (params.subtitle_scale < _input_subtitle_params.subtitle_scale
+                    || params.subtitle_scale > _input_subtitle_params.subtitle_scale)
+                || params.subtitle_color != _input_subtitle_params.subtitle_color))
+    {
+        // We have a new subtitle or a new video display size or new parameters,
+        // therefore we need to render the subtitle into _input_subtitle_tex.
+
+        // Regenerate an appropriate subtitle texture if necessary.
+        if (_input_subtitle_tex[index] == 0
+                || width != _input_subtitle_width[index]
+                || height != _input_subtitle_height[index])
+        {
+            if (_input_subtitle_tex[index] != 0)
+            {
+                glDeleteTextures(1, &(_input_subtitle_tex[index]));
+            }
+            glGenTextures(1, &(_input_subtitle_tex[index]));
+            glBindTexture(GL_TEXTURE_2D, _input_subtitle_tex[index]);
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8,
+                    width, height, 0,
+                    GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, NULL);
+        }
+        // Clear the texture
+        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, _input_fbo);
+        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,
+                GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _input_subtitle_tex[index], 0);
+        glClear(GL_COLOR_BUFFER_BIT);
+        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
+        // Prerender the subtitle to get a bounding box
+        int bb_x, bb_y, bb_w, bb_h;
+        _subtitle_renderer.prerender(subtitle, frame.presentation_time, params,
+                width, height, screen_pixel_aspect_ratio(),
+                bb_x, bb_y, bb_w, bb_h);
+        if (bb_w > 0 && bb_h > 0)
+        {
+            // Get a PBO buffer of appropriate size for the bounding box.
+            size_t size = bb_w * bb_h * sizeof(uint32_t);
+            glBindBuffer(GL_PIXEL_UNPACK_BUFFER, _input_pbo);
+            glBufferData(GL_PIXEL_UNPACK_BUFFER, size, NULL, GL_STREAM_DRAW);
+            void *pboptr = glMapBuffer(GL_PIXEL_UNPACK_BUFFER, GL_WRITE_ONLY);
+            if (!pboptr)
+            {
+                msg::err(_("Cannot create a PBO buffer."));
+                abort();
+            }
+            assert(reinterpret_cast<uintptr_t>(pboptr) % 4 == 0);
+            // Render the subtitle into the buffer.
+            _subtitle_renderer.render(static_cast<uint32_t *>(pboptr));
+            // Update the appropriate part of the texture.
+            glUnmapBuffer(GL_PIXEL_UNPACK_BUFFER);
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, bb_w);
+            glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
+            glActiveTexture(GL_TEXTURE0);
+            glBindTexture(GL_TEXTURE_2D, _input_subtitle_tex[index]);
+            glTexSubImage2D(GL_TEXTURE_2D, 0, bb_x, bb_y, bb_w, bb_h,
+                    GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, NULL);
+            glBindBuffer(GL_PIXEL_UNPACK_BUFFER, 0);
+        }
+    }
+    _input_subtitle_box[index] = subtitle;
+    _input_subtitle_width[index] = width;
+    _input_subtitle_height[index] = height;
+    _input_subtitle_time[index] = frame.presentation_time;
+    _input_subtitle_params = params;
+    assert(xgl::CheckError(HERE));
 }
 
 void video_output::color_init(const video_frame &frame)
@@ -533,6 +658,17 @@ void video_output::render_init()
     str::replace(render_fs_src, "$srgb_broken", srgb_broken_str);
     _render_prg = xgl::CreateProgram("video_output_render", "", "", render_fs_src);
     xgl::LinkProgram("video_output_render", _render_prg);
+    uint32_t dummy_texture = 0;
+    glGenTextures(1, &_render_dummy_tex);
+    glBindTexture(GL_TEXTURE_2D, _render_dummy_tex);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+    glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
+    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
+    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 1, 1, 0,
+            GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, &dummy_texture);
     if (_params.stereo_mode == parameters::even_odd_rows
             || _params.stereo_mode == parameters::even_odd_columns
             || _params.stereo_mode == parameters::checkerboard)
@@ -564,6 +700,11 @@ void video_output::render_deinit()
         xgl::DeleteProgram(_render_prg);
         _render_prg = 0;
     }
+    if (_render_dummy_tex != 0)
+    {
+        glDeleteTextures(1, &_render_dummy_tex);
+        _render_dummy_tex = 0;
+    }
     if (_render_mask_tex != 0)
     {
         glDeleteTextures(1, &_render_mask_tex);
@@ -610,8 +751,9 @@ static void draw_quad(float x, float y, float w, float h)
     glEnd();
 }
 
-void video_output::display_current_frame(bool mono_right_instead_of_left,
-            float x, float y, float w, float h, const GLint viewport[4])
+void video_output::display_current_frame(
+        bool keep_viewport, bool mono_right_instead_of_left,
+        float x, float y, float w, float h, const GLint viewport[2][4])
 {
     make_context_current();
     assert(xgl::CheckError(HERE));
@@ -622,11 +764,12 @@ void video_output::display_current_frame(bool mono_right_instead_of_left,
         return;
     }
 
-    if (frame.width != _color_last_frame.width
-            || frame.height != _color_last_frame.height
-            || frame.aspect_ratio < _color_last_frame.aspect_ratio
-            || frame.aspect_ratio > _color_last_frame.aspect_ratio
-            || _render_last_params.stereo_mode != _params.stereo_mode)
+    if (!keep_viewport
+            && (frame.width != _color_last_frame.width
+                || frame.height != _color_last_frame.height
+                || frame.aspect_ratio < _color_last_frame.aspect_ratio
+                || frame.aspect_ratio > _color_last_frame.aspect_ratio
+                || _render_last_params.stereo_mode != _params.stereo_mode))
     {
         reshape(width(), height());
     }
@@ -653,13 +796,13 @@ void video_output::display_current_frame(bool mono_right_instead_of_left,
     }
     if ((_params.stereo_mode == parameters::even_odd_rows
                 || _params.stereo_mode == parameters::checkerboard)
-            && (pos_y() + viewport[1]) % 2 == 0)
+            && (pos_y() + viewport[0][1]) % 2 == 0)
     {
         std::swap(left, right);
     }
     if ((_params.stereo_mode == parameters::even_odd_columns
                 || _params.stereo_mode == parameters::checkerboard)
-            && (pos_x() + viewport[0]) % 2 == 1)
+            && (pos_x() + viewport[0][0]) % 2 == 1)
     {
         std::swap(left, right);
     }
@@ -737,7 +880,7 @@ void video_output::display_current_frame(bool mono_right_instead_of_left,
         draw_quad(-1.0f, +1.0f, +2.0f, -2.0f);
     }
     glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
-    glViewport(viewport[0], viewport[1], viewport[2], viewport[3]);
+    glViewport(viewport[0][0], viewport[0][1], viewport[0][2], viewport[0][3]);
     glMatrixMode(GL_PROJECTION);
     glPopMatrix();
     glMatrixMode(GL_MODELVIEW);
@@ -752,7 +895,14 @@ void video_output::display_current_frame(bool mono_right_instead_of_left,
     right = (left != right ? 1 : 0);
     left = 0;
 
-    // Step 3: rendering
+    /* Step 3: rendering */
+
+    // Update the subtitle texture. This only re-renders the subtitle in the
+    // unlikely case that the video display area was resized between the call
+    // to prepare_next_frame and now (e.g. when resizing the window in pause
+    // mode while subtitles are displayed).
+    update_subtitle_tex(_active_index, frame, _input_subtitle_box[_active_index], _params);
+
     glUseProgram(_render_prg);
     glActiveTexture(GL_TEXTURE0);
     glBindTexture(GL_TEXTURE_2D, _color_srgb_tex[left]);
@@ -761,9 +911,14 @@ void video_output::display_current_frame(bool mono_right_instead_of_left,
         glActiveTexture(GL_TEXTURE1);
         glBindTexture(GL_TEXTURE_2D, _color_srgb_tex[right]);
     }
+    glActiveTexture(GL_TEXTURE2);
+    glBindTexture(GL_TEXTURE_2D, (_input_subtitle_box[_active_index].is_valid()
+                ? _input_subtitle_tex[_active_index] : _render_dummy_tex));
     glUniform1i(glGetUniformLocation(_render_prg, "rgb_l"), left);
     glUniform1i(glGetUniformLocation(_render_prg, "rgb_r"), right);
     glUniform1f(glGetUniformLocation(_render_prg, "parallax"), _params.parallax * 0.05f);
+    glUniform1i(glGetUniformLocation(_render_prg, "subtitle"), 2);
+    glUniform1f(glGetUniformLocation(_render_prg, "subtitle_parallax"), _params.subtitle_parallax * 0.05f);
     if (_params.stereo_mode != parameters::red_green_monochrome
             && _params.stereo_mode != parameters::red_cyan_half_color
             && _params.stereo_mode != parameters::red_cyan_full_color
@@ -788,9 +943,9 @@ void video_output::display_current_frame(bool mono_right_instead_of_left,
             || _params.stereo_mode == parameters::even_odd_columns
             || _params.stereo_mode == parameters::checkerboard)
     {
-        glUniform1i(glGetUniformLocation(_render_prg, "mask_tex"), 2);
-        glUniform1f(glGetUniformLocation(_render_prg, "step_x"), 1.0f / static_cast<float>(viewport[2]));
-        glUniform1f(glGetUniformLocation(_render_prg, "step_y"), 1.0f / static_cast<float>(viewport[3]));
+        glUniform1i(glGetUniformLocation(_render_prg, "mask_tex"), 3);
+        glUniform1f(glGetUniformLocation(_render_prg, "step_x"), 1.0f / static_cast<float>(viewport[0][2]));
+        glUniform1f(glGetUniformLocation(_render_prg, "step_y"), 1.0f / static_cast<float>(viewport[0][3]));
     }
 
     if (_params.stereo_mode == parameters::stereo)
@@ -806,9 +961,9 @@ void video_output::display_current_frame(bool mono_right_instead_of_left,
             || _params.stereo_mode == parameters::even_odd_columns
             || _params.stereo_mode == parameters::checkerboard)
     {
-        float vpw = static_cast<float>(viewport[2]);
-        float vph = static_cast<float>(viewport[3]);
-        glActiveTexture(GL_TEXTURE2);
+        float vpw = static_cast<float>(viewport[0][2]);
+        float vph = static_cast<float>(viewport[0][3]);
+        glActiveTexture(GL_TEXTURE3);
         glBindTexture(GL_TEXTURE_2D, _render_mask_tex);
         glBegin(GL_QUADS);
         glTexCoord2f(0.0f, 0.0f);
@@ -855,20 +1010,16 @@ void video_output::display_current_frame(bool mono_right_instead_of_left,
         draw_quad(x, y, w, h);
     }
     else if (_params.stereo_mode == parameters::left_right
-            || _params.stereo_mode == parameters::left_right_half)
+            || _params.stereo_mode == parameters::left_right_half
+            || _params.stereo_mode == parameters::top_bottom
+            || _params.stereo_mode == parameters::top_bottom_half
+            || _params.stereo_mode == parameters::hdmi_frame_pack)
     {
         glUniform1f(glGetUniformLocation(_render_prg, "channel"), 0.0f);
-        draw_quad(-1.0f, -1.0f, 1.0f, 2.0f);
+        draw_quad(x, y, w, h);
+        glViewport(viewport[1][0], viewport[1][1], viewport[1][2], viewport[1][3]);
         glUniform1f(glGetUniformLocation(_render_prg, "channel"), 1.0f);
-        draw_quad(0.0f, -1.0f, 1.0f, 2.0f);
-    }
-    else if (_params.stereo_mode == parameters::top_bottom
-            || _params.stereo_mode == parameters::top_bottom_half)
-    {
-        glUniform1f(glGetUniformLocation(_render_prg, "channel"), 0.0f);
-        draw_quad(-1.0f, 0.0f, 2.0f, 1.0f);
-        glUniform1f(glGetUniformLocation(_render_prg, "channel"), 1.0f);
-        draw_quad(-1.0f, -1.0f, 2.0f, 1.0f);
+        draw_quad(x, y, w, h);
     }
     assert(xgl::CheckError(HERE));
 }
@@ -891,14 +1042,36 @@ void video_output::clear()
     assert(xgl::CheckError(HERE));
 }
 
+static void compute_viewport(int vp[4], int w, int h, float dst_w, float dst_h, float dst_ar, float src_ar)
+{
+    vp[2] = dst_w;
+    vp[3] = dst_h;
+    if (src_ar >= dst_ar)
+    {
+        // need black borders top and bottom
+        vp[3] = dst_ar / src_ar * dst_h;
+    }
+    else
+    {
+        // need black borders left and right
+        vp[2] = src_ar / dst_ar * dst_w;
+    }
+    vp[0] = (w - vp[2]) / 2;
+    vp[1] = (h - vp[3]) / 2;
+}
+
 void video_output::reshape(int w, int h)
 {
     make_context_current();
     // Clear
-    _viewport[0] = 0;
-    _viewport[1] = 0;
-    _viewport[2] = w;
-    _viewport[3] = h;
+    _viewport[0][0] = 0;
+    _viewport[0][1] = 0;
+    _viewport[0][2] = w;
+    _viewport[0][3] = h;
+    _viewport[1][0] = 0;
+    _viewport[1][1] = 0;
+    _viewport[1][2] = w;
+    _viewport[1][3] = h;
     glViewport(0, 0, w, h);
     clear();
     if (!_frame[_active_index].is_valid())
@@ -907,38 +1080,72 @@ void video_output::reshape(int w, int h)
     }
 
     // Compute viewport with the right aspect ratio
-    float dst_w = w;
-    float dst_h = h;
-    float dst_ar = dst_w * screen_pixel_aspect_ratio() / dst_h;
-    float src_ar = _frame[_active_index].aspect_ratio;
-    if (_params.stereo_mode == parameters::left_right)
+    if (_params.stereo_mode == parameters::left_right
+            || _params.stereo_mode == parameters::left_right_half)
     {
-        src_ar *= 2.0f;
+        float dst_w = w / 2;
+        float dst_h = h;
+        float dst_ar = dst_w * screen_pixel_aspect_ratio() / dst_h;
+        float src_ar = _frame[_active_index].aspect_ratio;
+        if (_params.stereo_mode == parameters::left_right_half)
+        {
+            src_ar /= 2.0f;
+        }
+        compute_viewport(_viewport[0], w / 2, h, dst_w, dst_h, dst_ar, src_ar);
+        _viewport[1][0] = _viewport[0][0] + w / 2;
+        _viewport[1][1] = _viewport[0][1];
+        _viewport[1][2] = _viewport[0][2];
+        _viewport[1][3] = _viewport[0][3];
     }
-    else if (_params.stereo_mode == parameters::top_bottom)
+    else if (_params.stereo_mode == parameters::top_bottom
+            || _params.stereo_mode == parameters::top_bottom_half)
     {
-        src_ar /= 2.0f;
+        float dst_w = w;
+        float dst_h = h / 2;
+        float dst_ar = dst_w * screen_pixel_aspect_ratio() / dst_h;
+        float src_ar = _frame[_active_index].aspect_ratio;
+        if (_params.stereo_mode == parameters::top_bottom_half)
+        {
+            src_ar *= 2.0f;
+        }
+        compute_viewport(_viewport[0], w, h / 2, dst_w, dst_h, dst_ar, src_ar);
+        _viewport[1][0] = _viewport[0][0];
+        _viewport[1][1] = _viewport[0][1] + h / 2;
+        _viewport[1][2] = _viewport[0][2];
+        _viewport[1][3] = _viewport[0][3];
     }
-    int vp_w = w;
-    int vp_h = h;
-    if (src_ar >= dst_ar)
+    else if (_params.stereo_mode == parameters::hdmi_frame_pack)
     {
-        // need black borders top and bottom
-        vp_h = dst_ar / src_ar * dst_h;
+        // HDMI frame packing mode has left view top, right view bottom, plus a
+        // blank area separating the two. 720p uses 30 blank lines (total: 720
+        // + 30 + 720 = 1470), 1080p uses 45 (total: 10280 + 45 + 1080 = 2205).
+        // In both cases, the blank area is 30/1470 = 45/2205 = 1/49 of the
+        // total height. See the document "High-Definition Multimedia Interface
+        // Specification Version 1.4a Extraction of 3D Signaling Portion" from
+        // hdmi.org.
+        int blank_lines = h / 49;
+        float dst_w = w;
+        float dst_h = (h - blank_lines) / 2;
+        float dst_ar = dst_w * screen_pixel_aspect_ratio() / dst_h;
+        float src_ar = _frame[_active_index].aspect_ratio;
+        compute_viewport(_viewport[0], w, (h - blank_lines) / 2, dst_w, dst_h, dst_ar, src_ar);
+        _viewport[1][0] = _viewport[0][0];
+        _viewport[1][1] = _viewport[0][1] + (h - blank_lines) / 2 + blank_lines;
+        _viewport[1][2] = _viewport[0][2];
+        _viewport[1][3] = _viewport[0][3];
     }
     else
     {
-        // need black borders left and right
-        vp_w = src_ar / dst_ar * dst_w;
+        float dst_w = w;
+        float dst_h = h;
+        float dst_ar = dst_w * screen_pixel_aspect_ratio() / dst_h;
+        float src_ar = _frame[_active_index].aspect_ratio;
+        compute_viewport(_viewport[0], w, h, dst_w, dst_h, dst_ar, src_ar);
+        _viewport[1][0] = _viewport[0][0];
+        _viewport[1][1] = _viewport[0][1];
+        _viewport[1][2] = _viewport[0][2];
+        _viewport[1][3] = _viewport[0][3];
     }
-    int vp_x = (w - vp_w) / 2;
-    int vp_y = (h - vp_h) / 2;
-
-    // Save new size
-    _viewport[0] = vp_x;
-    _viewport[1] = vp_y;
-    _viewport[2] = vp_w;
-    _viewport[3] = vp_h;
 }
 
 bool video_output::need_redisplay_on_move()
