@@ -109,6 +109,18 @@ void video_output_qt_widget::keyPressEvent(QKeyEvent *event)
     switch (event->key())
     {
     case Qt::Key_Escape:
+        // ESC should stop playback, unless we're in fullscreen mode.
+        // In that case, it should simply leave fullscreen mode, since
+        // this is what most users expect.
+        if (_vo->_fullscreen)
+        {
+            _vo->send_cmd(command::toggle_fullscreen);
+        }
+        else
+        {
+            _vo->send_cmd(command::toggle_play);
+        }
+        break;
     case Qt::Key_Q:
         _vo->send_cmd(command::toggle_play);
         break;
@@ -283,24 +295,16 @@ void video_output_qt::init()
         create_widget();
         _widget->makeCurrent();
         set_opengl_versions();
-        try
+        GLenum err = glewInit();
+        if (err != GLEW_OK)
         {
-            GLenum err = glewInit();
-            if (err != GLEW_OK)
-            {
-                throw exc(str::asprintf(_("Cannot initialize GLEW: %s"),
-                            reinterpret_cast<const char *>(glewGetErrorString(err))));
-            }
-            if (!glewIsSupported("GL_VERSION_2_1 GL_EXT_framebuffer_object"))
-            {
-                throw exc(std::string(_("This OpenGL implementation does not support "
-                                "OpenGL 2.1 and framebuffer objects.")));
-            }
+            throw exc(str::asprintf(_("Cannot initialize GLEW: %s"),
+                        reinterpret_cast<const char *>(glewGetErrorString(err))));
         }
-        catch (std::exception &e)
+        if (!glewIsSupported("GL_VERSION_2_1 GL_EXT_framebuffer_object"))
         {
-            QMessageBox::critical(_widget, _("Error"), e.what());
-            abort();
+            throw exc(std::string(_("This OpenGL implementation does not support "
+                            "OpenGL 2.1 and framebuffer objects.")));
         }
         video_output::init();
         video_output::clear();
@@ -533,11 +537,6 @@ void video_output_qt::exit_fullscreen()
     }
 }
 
-bool video_output_qt::has_events()
-{
-    return QApplication::hasPendingEvents();
-}
-
 void video_output_qt::process_events()
 {
     QApplication::sendPostedEvents();
